'====================================================================
'
'  Preview.inc for PowerBASIC Compiler for Windows.
'  Print Preview dialog include file.
'  Copyright (c) 2005 - 2008 PowerBASIC, Inc.
'  All Rights Reserved.
'
'--------------------------------------------------------------------
'
'  You have a royalty-free right to use, modify, reproduce and distribute
'  this file (and/or any modified version) in any way you find useful,
'  provided that you agree that both PowerBASIC and Microsoft have no
'  warranty, obligation or liability for its contents.
'
'--------------------------------------------------------------------
'
'  This dialog provides text Printing and Print Preview features for
'  any program by simply inserting #INCLUDE "PREVIEW.INC" among the
'  declares for the main dialog and then call like:
'
'  DlgPreview hDlg, sTitle, sText, pd
'
'    hDlg   = Parent dialog's handle.
'    sTitle = File path+name.
'    sText  = Text to Print/Preview.
'    pd     = PreviewData variable.
'
'  Note: The main program is responsible for maintaining a Static
'        or Global PreviewData variable and call InitPreviewDlg.
'        See TextView.bas for an example of how-to-use.
'
'--------------------------------------------------------------------
'  Short desciption:
'
'  The Preview dialog uses two Graphic controls placed in a container
'  control, which in is reality is a child dialog with %DS_CONTROL
'  style. One Graphic control acts as a hidden, full-sized buffer in
'  which all printing is done. The other Graphic control acts as view
'  control with different sizes depending on zoom level. On resizing
'  or page change, the view control is killed and re-created with the
'  new size, while the buffer control is repainted and then stretched
'  into the visible view control. The darkgray container window is
'  resized to look as a full "client" area for the main Preview dialog,
'  and it handles scrollbars plus mouse scrolling.
'
'--------------------------------------------------------------------
'  Procedures:
'  DlgPreview      - Main function where dialog and ViewPort is created.
'  DlgPreviewProc  - Preview dialog callback procedure.
'  InitPreviewBuf  - Setup and create a Graphic buffer control.
'  InitPreviewDlg  - Get printer data and setup initial margins, etc.
'  PrepareText     - Put text into an array and line wrap at given width.
'  PrintToBuffer   - Prepare a page in the Graphic buffer control.
'  PrintToPrinter  - Print the entire text after a Print dialog is shown.
'  PrintToScreen   - Stretch the Graphic buffer to the screen.
'  SetScrollPosHV  - Set the ViewPort scrolling on zoom and resize.
'  SetScrollSize   - Set the ViewPort scrollbars on zoom and resize.
'  ViewPortProc    - ViewPort parent dialog's callback procedure.
'  vp_OnHScroll    - %WM_HSCROLL handler for the ViewPort.
'  vp_OnMouseWheel - %WM_MOUSEWHEEL handler for the ViewPort.
'  vp_OnVScroll    - %WM_VSCROLL handler for the ViewPort.
'
'====================================================================
' Declares
'----------------------------------------------------------
#IF NOT %DEF(%WINAPI)       ' if not already included
  %USEMACROS = 1            ' Use Macros for smallest size
  #INCLUDE "WIN32API.INC"   ' Win32 API declares
#ENDIF

'----------------------------------------------------------
%IDC_GRAPHICBUFF  = %WM_USER + 101
%IDC_GRAPHICVIEW  = %WM_USER + 102
%IDC_LABEL1       = %WM_USER + 111
%IDC_LABEL2       = %WM_USER + 112
%IDC_BUTNPRINT    = %WM_USER + 121
%IDC_BUTNFIRST    = %WM_USER + 122
%IDC_BUTNPREV     = %WM_USER + 123
%IDC_BUTNNEXT     = %WM_USER + 124
%IDC_BUTNLAST     = %WM_USER + 125
%IDC_BUTNZOOM     = %WM_USER + 126
'------------------------------------------------
TYPE PreviewData       ' SIZEOF = 240 bytes
  pw        AS SINGLE  ' Printer page width
  ph        AS SINGLE  ' Printer page height
  mL        AS SINGLE  ' Left   margin
  mT        AS SINGLE  ' Top    margin
  mR        AS SINGLE  ' Right  margin
  mB        AS SINGLE  ' Bottom margin
  lm        AS SINGLE  ' Left   non-printing region
  tm        AS SINGLE  ' Top    non-printing region
  rm        AS SINGLE  ' Right  non-printing region
  bm        AS SINGLE  ' Bottom non-printing region
  orient    AS LONG    ' Printer orientation
  pXppi     AS LONG    ' Printer resolution x
  pYppi     AS LONG    ' Printer resolution y
  '----------------------------------------------
  gw        AS SINGLE  ' Graphic view page width
  gh        AS SINGLE  ' Graphic view page height
  gx1       AS SINGLE  ' Graphic view Left   margin
  gy1       AS SINGLE  ' Graphic view Top    margin
  gx2       AS SINGLE  ' Graphic view Right  margin
  gy2       AS SINGLE  ' Graphic view Bottom margin
  '----------------------------------------------
  gpw       AS SINGLE  ' Graphic buffer page width
  gph       AS SINGLE  ' Graphic buffer page height
  gcw       AS SINGLE  ' Graphic buffer client width
  gch       AS SINGLE  ' Graphic buffer client height
  gmL       AS SINGLE  ' Graphic buffer Left   margin
  gmT       AS SINGLE  ' Graphic buffer Top    margin
  gmR       AS SINGLE  ' Graphic buffer Right  margin
  gmB       AS SINGLE  ' Graphic buffer Bottom margin
  '----------------------------------------------
  gXppi     AS LONG    ' Graphic resolution x
  gYppi     AS LONG    ' Graphic resolution y
  locale    AS LONG    ' Locale measurement, US/Metric - 1/0
  lpp       AS LONG    ' Lines per page
  PageCount AS LONG    ' Total page count
  zoom      AS LONG    ' zoom factor, 0 - 100
  zoomX     AS SINGLE  ' zoom factor x
  zoomY     AS SINGLE  ' zoom factor y
  '----------------------------------------------
  clrWindow     AS LONG   ' System window color
  clrWindowText AS LONG   ' System window text color
  clrShadow     AS LONG   ' System shadow color
  clrDkShadow   AS LONG   ' System dark shadow color
  clrBorder     AS LONG   ' Page border color (def blue)
  FontSize      AS LONG   ' Font size
  FontType      AS LONG   ' Font type
  FontName AS ASCIIZ * 64 ' Font name
  pt       AS POINTAPI    ' Cursor pos on zoom click
END TYPE
'--------------------------------------------------------------------
DECLARE SUB InitPreviewBuf (BYVAL hDlg AS DWORD)
DECLARE SUB InitPreviewDlg (pd AS PreviewData)
DECLARE SUB PrepareText    (BYVAL hDlg AS DWORD, BYVAL w AS SINGLE, _
                            BYVAL sText AS STRING, sArray() AS STRING)
DECLARE SUB PrintToBuffer  (BYVAL hDlg AS DWORD, BYVAL PageNum AS LONG, sArray() AS STRING)
DECLARE SUB PrintToPrinter (BYVAL hDlg AS DWORD, sArray() AS STRING)
DECLARE SUB PrintToScreen  (BYVAL hDlg AS DWORD, sArray() AS STRING)
DECLARE SUB SetScrollPosHV (BYVAL hDlg AS DWORD)
DECLARE SUB SetScrollSize  (BYVAL hDlg AS DWORD, BYVAL w AS LONG, BYVAL h AS LONG)
'--------------------------------------------------------------------
DECLARE FUNCTION vp_OnHScroll     (BYVAL hWnd AS DWORD, _
                                   BYVAL wParam AS DWORD, _
                                   BYVAL lParam AS LONG) AS LONG
DECLARE FUNCTION vp_OnVScroll     (BYVAL hWnd AS DWORD, _
                                   BYVAL wParam AS DWORD, _
                                   BYVAL lParam AS LONG) AS LONG
DECLARE FUNCTION vp_OnMouseWheel  (BYVAL hWnd AS DWORD, _
                                   BYVAL wParam AS DWORD, _
                                   BYVAL lParam AS LONG) AS LONG


'====================================================================
FUNCTION DlgPreview (BYVAL hDlgMain AS DWORD, _      ' Parent dialog handle
                     BYVAL sTitle AS STRING, _       ' File path+name
                     BYVAL sText AS STRING, _        ' Text to print/preview
                     pd AS PreviewData) AS LONG      ' Printer and Preview data
'--------------------------------------------------------------------
' Program entry point
'--------------------------------------------------------------------
  LOCAL hDlg, hClient AS DWORD, h, w, x, y AS LONG

  '------------------------------------------------------------------
  ' Let initial size be 90% of the desktop client size.
  '------------------------------------------------------------------
  DESKTOP GET CLIENT TO w, h
  w = w * 0.9
  h = h * 0.9

  '------------------------------------------------------------------
  ' Use a PIXELS based dialog for more exact measurements
  '------------------------------------------------------------------
  DIALOG NEW PIXELS, hDlgMain, sTitle,,, w, h, _
                     %WS_OVERLAPPEDWINDOW OR %WS_CLIPCHILDREN, 0 TO hDlg

  '------------------------------------------------------------------
  ' Use a Label as "ToolBar"
  '------------------------------------------------------------------
  DIALOG UNITS hDlg, 44, 16 TO PIXELS w, h
  CONTROL ADD LABEL, hDlg, %IDC_LABEL1, "", 0, 0, 640, h,, %WS_EX_DLGMODALFRAME


  '------------------------------------------------------------------
  ' ViewPort, a dialog with %DS_CONTROL style acting as Graphic parent.
  '------------------------------------------------------------------
  DIALOG NEW PIXELS, hDlg, "", 0, h, 50, 50, %DS_CONTROL OR _ ' Scroll window
                     %WS_CHILD OR %WS_CLIPCHILDREN, 0 TO hClient
  DIALOG SET COLOR hClient, -1, GetSysColor(%COLOR_3DSHADOW)
  '------------------------------------------------------------------
  CONTROL ADD GRAPHIC, hClient, %IDC_GRAPHICVIEW, "", 0, 0, 50, 50, %SS_NOTIFY
  GRAPHIC ATTACH hClient, %IDC_GRAPHICVIEW, REDRAW


  '------------------------------------------------------------------
  ' Add buttons and a label to the "ToolBar"
  '------------------------------------------------------------------
  DIALOG UNITS hDlg, 44, 12 TO PIXELS w, h  ' Button size
  DIALOG UNITS hDlg, 2,   2 TO PIXELS x, y  ' Space between..

  CONTROL ADD BUTTON, hDlg, %IDC_BUTNPRINT, "&Print", x, 3, w, h
  x = x + w + 4*y
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNFIRST, "<<",  x, 3, w/2, h
  x = x + w/2
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNPREV,  "<",   x, 3, w/2, h
  x = x + w/2
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNNEXT,  ">",   x, 3, w/2, h
  x = x + w/2
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNLAST,  ">>",  x, 3, w/2, h
  x = x + w/2 + y
  '------------------------------------------------------------------
  CONTROL ADD LABEL, hDlg, %IDC_LABEL2, "", x, 7, w, h - 2, %SS_CENTER
  x = x + w + y
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNZOOM, "&Zoom", x, 3, w, h
  x = x + w + 4*y
  CONTROL ADD BUTTON, hDlg, %IDCANCEL, "&Close", x, 3, w, h

  '------------------------------------------------------------------
  CONTROL DISABLE hDlg, %IDC_BUTNFIRST
  CONTROL DISABLE hDlg, %IDC_BUTNPREV
  CONTROL DISABLE hDlg, %IDC_BUTNNEXT
  CONTROL DISABLE hDlg, %IDC_BUTNLAST

  '------------------------------------------------------------------
  ' Store some important handles and pointer to given text with USER
  '------------------------------------------------------------------
  IF LEN(sText) = 0 THEN sText = " "
  DIALOG SET USER hDlg, 1, hClient
  DIALOG SET USER hDlg, 2, VARPTR(pd)
  DIALOG SET USER hDlg, 3, STRPTR(sText)
  DIALOG SET USER hDlg, 4, LEN(sText)
  DIALOG SET USER hClient, 1, hDlg
  DIALOG SET USER hClient, 2, VARPTR(pd)

  '------------------------------------------------------------------
  DIALOG SHOW MODELESS hClient, CALL ViewPortProc
  DIALOG SHOW MODAL hDlg CALL DlgPreviewProc

END FUNCTION


'====================================================================
CALLBACK FUNCTION DlgPreviewProc() AS LONG
'--------------------------------------------------------------------
' Main dialog callback procedure
'--------------------------------------------------------------------
  LOCAL w, x, y AS LONG, hClient, pStr AS DWORD
  LOCAL zLocale AS ASCIIZ * 4
  LOCAL pd AS PreviewData PTR
  '----------------------------------------------
  STATIC sArray() AS STRING
  STATIC hCurZoom AS DWORD
  STATIC noFlag, PageNum AS LONG
  STATIC sText AS STRING
  '----------------------------------------------
  DIALOG GET USER CB.HNDL, 1 TO hClient
  DIALOG GET USER CB.HNDL, 2 TO pd
  '----------------------------------------------

  SELECT CASE CB.MSG
  CASE %WM_INITDIALOG  ' <- Received right before the dialog is shown
      '--------------------------------------------------------------
      hCurZoom  = LoadCursor(GetModuleHandle(""), "ZOOM")
      '--------------------------------------------------------------
      DIALOG GET USER CB.HNDL, 3 TO pStr
      DIALOG GET USER CB.HNDL, 4 TO w
      IF w THEN sText = PEEK$(pStr, w)
      '--------------------------------------------------------------

      GRAPHIC FONT @pd.FontName, @pd.FontSize, @pd.FontType
      GRAPHIC GET PPI TO @pd.gXppi, @pd.gYppi  ' Graphic resolution in pixels/inch

      '--------------------------------------------------------------
      ' Initialize and show first page
      '--------------------------------------------------------------
      InitPreviewBuf hClient

      '--------------------------------------------------------------
      ' Calculate lines-per-page
      '--------------------------------------------------------------
      GRAPHIC CHR SIZE TO x, y   ' Line height
      @pd.lpp = @pd.gch \ y      ' lines-per-page

      '--------------------------------------------------------------
      ' Setup and show Page 1
      '--------------------------------------------------------------
      IF LEN(sText) THEN
          PageNum = 1
          PrepareText   hClient, @pd.gcw, sText, sArray()
          PrintToBuffer hClient, PageNum, sArray()
          PrintToScreen hClient, sArray()

          IF @pd.PageCount > 1 THEN
              CONTROL ENABLE CB.HNDL, %IDC_BUTNNEXT
              CONTROL ENABLE CB.HNDL, %IDC_BUTNLAST
          END IF
      END IF
      '--------------------------------------------------------------
      CONTROL SET FOCUS CB.HNDL, %IDCANCEL
      FUNCTION = 1


  CASE %WM_SIZE
      '--------------------------------------------------------------
      ' Auto-size the ToolBar label and the Graphic ViewPort parent.
      '--------------------------------------------------------------
      IF CB.WPARAM <> %SIZE_MINIMIZED THEN
          CONTROL GET SIZE CB.HNDL, %IDC_LABEL1 TO x, y
          CONTROL SET SIZE CB.HNDL, %IDC_LABEL1, LO(WORD, CB.LPARAM) + 2, y
          DIALOG SET SIZE hClient, LO(WORD, CB.LPARAM), HI(WORD, CB.LPARAM) - y
          PrintToScreen hClient, sArray()
      END IF


  CASE %WM_GETMINMAXINFO
      '--------------------------------------------------------------
      ' Prevent too small dialog size
      '--------------------------------------------------------------
      LOCAL mmi AS MINMAXINFO PTR
      mmi = CB.LPARAM
      @mmi.ptMinTrackSize.x = 200
      @mmi.ptMinTrackSize.y = 200
      FUNCTION = 1


  CASE %WM_COMMAND         ' <- a control is calling
      SELECT CASE CB.CTL    ' <- look at control's id
      CASE %IDC_BUTNPRINT  ' "Print" button was clicked
          IF CB.CTLMSG = %BN_CLICKED THEN
              PrintToPrinter CB.HNDL, sArray()
          END IF

      CASE %IDC_BUTNFIRST  ' "<<" button was clicked
          IF CB.CTLMSG = %BN_CLICKED THEN
              PageNum = 1
              CONTROL DISABLE CB.HNDL, %IDC_BUTNFIRST
              CONTROL DISABLE CB.HNDL, %IDC_BUTNPREV
              IF @pd.PageCount > 1 THEN
                  CONTROL ENABLE CB.HNDL, %IDC_BUTNNEXT
                  CONTROL ENABLE CB.HNDL, %IDC_BUTNLAST
              END IF
              CONTROL SET FOCUS CB.HNDL, %IDC_BUTNNEXT
              PrintToBuffer hClient, PageNum, sArray()
              PrintToScreen hClient, sArray()
          END IF

      CASE %IDC_BUTNPREV  ' "<" button was clicked
          IF CB.CTLMSG = %BN_CLICKED THEN
              PageNum = MAX&(1, PageNum - 1)
              CONTROL ENABLE CB.HNDL, %IDC_BUTNNEXT
              CONTROL ENABLE CB.HNDL, %IDC_BUTNLAST
              IF PageNum = 1 THEN
                  CONTROL DISABLE CB.HNDL, %IDC_BUTNFIRST
                  CONTROL DISABLE CB.HNDL, %IDC_BUTNPREV
                  CONTROL SET FOCUS CB.HNDL, %IDC_BUTNNEXT
              END IF
              PrintToBuffer hClient, PageNum, sArray()
              PrintToScreen hClient, sArray()
          END IF

      CASE %IDC_BUTNNEXT  ' ">" button was clicked
          IF CB.CTLMSG = %BN_CLICKED THEN
              PageNum = MIN&(MAX&(1, @pd.PageCount), PageNum + 1)
              CONTROL ENABLE CB.HNDL, %IDC_BUTNFIRST
              CONTROL ENABLE CB.HNDL, %IDC_BUTNPREV
              IF PageNum = @pd.PageCount THEN
                  CONTROL DISABLE CB.HNDL, %IDC_BUTNNEXT
                  CONTROL DISABLE CB.HNDL, %IDC_BUTNLAST
                  CONTROL SET FOCUS CB.HNDL, %IDC_BUTNPREV
              END IF
              PrintToBuffer hClient, PageNum, sArray()
              PrintToScreen hClient, sArray()
          END IF

      CASE %IDC_BUTNLAST  ' ">>" button was clicked
          IF CB.CTLMSG = %BN_CLICKED THEN
              PageNum = MAX&(1, @pd.PageCount)
              CONTROL ENABLE CB.HNDL, %IDC_BUTNFIRST
              CONTROL ENABLE CB.HNDL, %IDC_BUTNPREV
              IF PageNum = @pd.PageCount THEN
                  CONTROL DISABLE CB.HNDL, %IDC_BUTNNEXT
                  CONTROL DISABLE CB.HNDL, %IDC_BUTNLAST
              END IF
              CONTROL SET FOCUS CB.HNDL, %IDC_BUTNPREV
              PrintToBuffer hClient, PageNum, sArray()
              PrintToScreen hClient, sArray()
          END IF

      CASE %IDC_BUTNZOOM
          IF CB.CTLMSG = %BN_CLICKED THEN
              SELECT CASE @pd.zoom
              CASE 0   : @pd.zoom = 75
              CASE 75  : @pd.zoom = 100
              CASE 100 : @pd.zoom = 0
              END SELECT
              '--------------------------------------------------
              PrintToScreen hClient, sArray()
          END IF

      CASE %IDCANCEL      ' "Cancel" button or the Esc-key was pressed
          IF CB.CTLMSG = %BN_CLICKED THEN ' close dialog
              DIALOG END CB.HNDL
          END IF

      END SELECT


  CASE %WM_SETCURSOR
      '------------------------------------------------------------------------
      ' When the mouse is over the Graphic control, over-ride default arrow
      ' cursor and set a magnify glass cursor instead. Note - in dialogs,
      ' we must return %TRUE to inform dialog engine we have taken charge.
      '------------------------------------------------------------------------
      IF GetDlgCtrlID(CB.WPARAM) = %IDC_GRAPHICVIEW THEN

          SELECT CASE AS LONG HI(WORD, CB.LPARAM)
          CASE %WM_LBUTTONDOWN                ' Zoom on clicks
              IF noFlag THEN EXIT FUNCTION    ' Avoid multiple clicks
              noFlag = 1
              GetCursorPos @pd.pt
              ScreenToClient hClient, @pd.pt  ' Store click-point
              SELECT CASE @pd.zoom            ' Toggle zoom value
              CASE 0   : @pd.zoom = 75
              CASE 75  : @pd.zoom = 100
              CASE 100 : @pd.zoom = 0
              END SELECT
              '--------------------------------------------------
              PrintToScreen hClient, sArray()
              noFlag = 0

          CASE %WM_MOUSEMOVE  ' Ensure zoom cursor is used
              IF GetCursor <> hCurZoom THEN SetCursor hCurZoom
          END SELECT

          FUNCTION = 1
          EXIT FUNCTION
      END IF


  CASE %WM_MOUSEWHEEL  ' Pass on mouse scroll messages
      FUNCTION = vp_OnMouseWheel (hClient, CB.WPARAM, CB.LPARAM)

  END SELECT
END FUNCTION


'====================================================================
SUB InitPreviewBuf (BYVAL hClient AS DWORD)
'--------------------------------------------------------------------
' Setup and create a Graphic buffer control
'--------------------------------------------------------------------
  LOCAL pd AS PreviewData PTR
  '------------------------------------------------------------------
  DIALOG GET USER hClient, 2 TO pd    ' Get preview data
  IF pd = 0 THEN EXIT SUB             ' Ensure we got something

  '--------------------------------------------------------------
  @pd.zoom = 0                                        ' Auto-sized to dialog height
  '--------------------------------------------------------------
  @pd.clrWindow     = GetSysColor(%COLOR_WINDOW)      ' System window color
  @pd.clrWindowText = GetSysColor(%COLOR_WINDOWTEXT)  ' System window text color
  @pd.clrShadow     = GetSysColor(%COLOR_3DSHADOW)    ' System shadow color
  @pd.clrDkShadow   = GetSysColor(%COLOR_3DDKSHADOW)  ' System dark shadow color
  @pd.clrBorder     = RGB(49,105,198)                 ' Blue-ish border

  '--------------------------------------------------------------
  ' if no printer resolution - set own page size and orientation
  '--------------------------------------------------------------
  IF @pd.pXppi = 0 THEN
      @pd.pXppi = @pd.gXppi                   ' Use Graphic resolution in pixels/inch
      @pd.pYppi = @pd.gYppi
      IF @pd.locale = 1 THEN                  ' US/Inches
          @pd.pw = 8.5 * @pd.pXppi             ' "Letter" page width in Graphic resolution
          @pd.ph = 11  * @pd.pYppi             ' "Letter" page height in Graphic resolution
      ELSE                                   ' Metric/millimeter
          @pd.pw = 21   / 2.54 * @pd.pXppi     ' "A4" page width in Graphic resolution
          @pd.ph = 29.7 / 2.54 * @pd.pYppi     ' "A4" page height in Graphic resolution
      END IF
      @pd.orient = 1          ' Set Portrait orientation mode
  END IF

  '--------------------------------------------------------------
  ' Calculate Graphic buffer size and margins
  '--------------------------------------------------------------
  @pd.gpw = @pd.pw * (@pd.gXppi / @pd.pXppi)  ' Graphic 100% page width
  @pd.gph = @pd.ph * (@pd.gYppi / @pd.pYppi)  ' Graphic 100% page height
  @pd.gmL = @pd.mL * (@pd.gXppi / 25.40)      ' Graphic 100% left margin
  @pd.gmT = @pd.mT * (@pd.gYppi / 25.40)      ' Graphic 100% top margin
  @pd.gmR = @pd.mR * (@pd.gXppi / 25.40)      ' Graphic 100% right margin
  @pd.gmB = @pd.mB * (@pd.gYppi / 25.40)      ' Graphic 100% bottom margin
  @pd.gcw = @pd.gpw - @pd.gmL - @pd.gmR + 2   ' Graphic 100% client width
  @pd.gch = @pd.gph - @pd.gmT - @pd.gmB + 2   ' Graphic 100% client height

  '--------------------------------------------------------------
  ' Create and setup Graphic buffer control
  '--------------------------------------------------------------
  CONTROL ADD GRAPHIC, hClient, %IDC_GRAPHICBUFF, "", 0, 0, @pd.gpw, @pd.gph
  CONTROL SHOW STATE hClient, %IDC_GRAPHICBUFF, %SW_HIDE
  GRAPHIC ATTACH hClient, %IDC_GRAPHICBUFF
  GRAPHIC COLOR @pd.clrWindowText, @pd.clrWindow
  GRAPHIC FONT @pd.FontName, @pd.FontSize, @pd.FontType
  GRAPHIC CLEAR

END SUB


'====================================================================
SUB InitPreviewDlg (pd AS PreviewData)
'--------------------------------------------------------------------
' Grab Printer page size and resolution, etc, plus
' setup initial margins, font, zoom and colors. To be
' called from Main program before the Preview is shown.
'--------------------------------------------------------------------
  LOCAL zLocale AS ASCIIZ * 4

  '----------------------------------------------------------------
  ERRCLEAR
  XPRINT ATTACH DEFAULT                     ' Attach the default host printer
  IF ERR = 0 AND LEN(XPRINT$) THEN          ' On success
      XPRINT GET SIZE TO pd.pw, pd.ph       ' Page size in pixels
      XPRINT GET MARGIN TO pd.lm, pd.tm, pd.rm, pd.bm  ' Printer margins (non-printable area)
      XPRINT GET ORIENTATION TO pd.orient   ' Printer orientation (1/2 - portrait/landscape)
      XPRINT GET PPI TO pd.pXppi, pd.pYppi  ' Get printer resolution in pixels/inch
      XPRINT CLOSE                          ' Detach the printer
  END IF

  '------------------------------------------------------------------
  ' Retrieve the locale measurement - Metric millimeters or inches.
  ' Returns 0 = Metrics or 1 = US (Inches)
  '--------------------------------------------------------------
  GetLocaleInfo %LOCALE_USER_DEFAULT, %LOCALE_IMEASURE, zLocale, SIZEOF(zLocale)
  pd.locale = VAL(zLocale)
  '--------------------------------------------------------------
  IF pd.locale = 1 THEN  ' US/Inches
      pd.mL   = 20.32    ' 20.32 mm margins = 0.8 inches
      pd.mT   = 20.32
      pd.mR   = 20.32
      pd.mB   = 20.32
  ELSE                   ' Metric/millimeters
      pd.mL   = 20       ' 20 mm margins
      pd.mT   = 20
      pd.mR   = 20
      pd.mB   = 20
  END IF
  '--------------------------------------------------------------
  pd.FontName = "Courier New"  ' Initial font name and size
  pd.FontSize = 10
  pd.FontType = 0
  '--------------------------------------------------------------

END SUB


'====================================================================
SUB PrepareText (BYVAL hClient AS DWORD, BYVAL w AS SINGLE, _
                 BYVAL sText AS STRING, sArray() AS STRING)
'--------------------------------------------------------------------
' Prepare given text for the preview and wrap too long lines
'--------------------------------------------------------------------
  LOCAL c, lCount, p, tw, th AS LONG
  LOCAL pd AS PreviewData PTR

  '------------------------------------------------------------------
  DIALOG GET USER hClient, 2 TO pd    ' Get preview data
  IF pd = 0 THEN EXIT SUB             ' Ensure we got something

  '--------------------------------------------
  ' Parse given text line-by-line into an array
  '--------------------------------------------
  IF LEN(sText) THEN
      IF RIGHT$(sText, 2) <> $CRLF THEN sText = sText + $CRLF
      lCount = TALLY(sText, $CRLF)
      IF lCount > 0 THEN
          REDIM sArray(1 TO lCount) AS STRING
          PARSE sText, sArray(), $CRLF
      END IF
  ELSEIF UBOUND(sArray) < 0 THEN
      EXIT SUB
  END IF

  '--------------------------------------------
  ' Ensure the Graphic buffer is attached
  '--------------------------------------------
  GRAPHIC ATTACH hClient, %IDC_GRAPHICBUFF
  GRAPHIC FONT @pd.FontName, @pd.FontSize, @pd.FontType

  '--------------------------------------------------------------
  ' Loop through all array items (lines) and compare their width
  ' with given page client width - if too long, wrap to next line.
  '--------------------------------------------------------------
  c = 1
  DO
      GRAPHIC TEXT SIZE sArray(c) TO tw, th  ' Line width
      IF tw > w THEN                         ' If too wide
          p = 1
          DO                                 ' Scan the line backwards
              p = INSTR(-p, sArray(c), " ")  ' for a space to wrap at
              IF p THEN
                  GRAPHIC TEXT SIZE LEFT$(sArray(c), p) TO tw, th
                  IF tw <= w THEN
                      REDIM PRESERVE sArray(1 TO UBOUND(sArray)+1)
                      ARRAY INSERT sArray(c), LEFT$(sArray(c), p)
                      sArray(c+1) = MID$(sArray(c+1), p+1)
                      DECR c
                      EXIT DO
                  ELSE
                      p = LEN(sArray(c)) - p + 2
                  END IF
              ELSE
                  FOR p = LEN(sArray(c)) - 1 TO 1 STEP -1
                      GRAPHIC TEXT SIZE LEFT$(sArray(c), p) TO tw, th
                      IF tw <= w THEN
                          sArray(c) = LEFT$(sArray(c), p)
                          EXIT FOR
                      END IF
                  NEXT
                  EXIT DO
              END IF
          LOOP
      END IF
      INCR c
      IF c > UBOUND(sArray) THEN EXIT DO
   LOOP
   @pd.PageCount = UBOUND(sArray) \ @pd.lpp + 1        ' and total PageCount

END SUB


'====================================================================
SUB PrintToBuffer (BYVAL hClient AS DWORD, _
                   BYVAL PageNum AS LONG, _
                   sArray() AS STRING)
'--------------------------------------------------------------------
' Print sArray() to the Graphic Buffer control
'--------------------------------------------------------------------
  LOCAL hDlgMain AS DWORD
  LOCAL c, x, y, StartLine, EndLine AS LONG
  LOCAL sHeader, sTitle AS STRING
  LOCAL pd AS PreviewData PTR

  '--------------------------------------------
  ' Get preview data, setup font and colors
  '--------------------------------------------
  DIALOG GET USER hClient, 1 TO hDlgMain
  DIALOG GET USER hClient, 2 TO pd
  IF pd = 0 THEN EXIT SUB

  '--------------------------------------------
  sHeader = "Page" + STR$(PageNum) + "/" + FORMAT$(@pd.PageCount)
  CONTROL SET TEXT hDlgMain, %IDC_LABEL2, sHeader

  '--------------------------------------------
  ' Attach the Graphic control
  '--------------------------------------------
  GRAPHIC ATTACH hClient, %IDC_GRAPHICBUFF
  GRAPHIC COLOR @pd.clrWindowText, @pd.clrWindow
  GRAPHIC CLEAR

  '--------------------------------------------
  ' Print a centered header
  '--------------------------------------------
  DIALOG GET TEXT hDlgMain TO sTitle
  c = INSTR(-1, sTitle, ANY "\/")
  IF c THEN
      sTitle = TRIM$(MID$(sTitle, c+1))
  ELSE
      sTitle = "Untitled"
  END IF
  sTitle = sTitle + "  |  " + sHeader + "  |  " + DATE$ + ", " + TIME$
  GRAPHIC COLOR @pd.clrDkShadow, @pd.clrWindow
  GRAPHIC TEXT SIZE sTitle TO x, y
  GRAPHIC SET POS ((@pd.gpw - x) / 2, (@pd.gmT - y) / 2)
  GRAPHIC PRINT sTitle

  '--------------------------------------------
  ' Print the text
  '--------------------------------------------
  StartLine = 1 + ((PageNum - 1) * @pd.lpp)
  StartLine = MIN&(StartLine, UBOUND(sArray))
  EndLine   = StartLine + @pd.lpp - 1
  EndLine   = MIN&(EndLine, UBOUND(sArray))

  GRAPHIC COLOR -1, -2
  GRAPHIC SET POS (@pd.gmL, @pd.gmT)  ' starting point
  FOR c = StartLine TO EndLine
      GRAPHIC GET POS TO x, y
      GRAPHIC SET POS (@pd.gmL, y)
      GRAPHIC PRINT sArray(c)
  NEXT

END SUB


'====================================================================
SUB PrintToPrinter (BYVAL hDlg AS DWORD, sArray() AS STRING)
'--------------------------------------------------------------------
' Print the entire text array after a Print dialog is shown.
'--------------------------------------------------------------------
  LOCAL c, x, y, x1, y1, x2, y2, w1, h1, pgNum AS LONG
  LOCAL sHeader  AS STRING, sTitle AS STRING
  LOCAL hFont AS LONG

  LOCAL pd AS PreviewData PTR
  '------------------------------------------------------------------
  DIALOG GET USER hDlg, 2 TO pd       ' Get preview data
  IF pd = 0 THEN EXIT SUB             ' Ensure we got something

  '----------------------------------------------------------------------------
  ' Document title for the header
  '----------------------------------------------------------------------------
  DIALOG GET TEXT hDlg TO sTitle
  c = INSTR(-1, sTitle, ANY "\/")
  IF c THEN
      sTitle = TRIM$(MID$(sTitle, c+1))
  ELSE
      sTitle = "Untitled"
  END IF

  '----------------------------------------------------------------------------
  ' Print the entire, line wrapped array
  '----------------------------------------------------------------------------
  ERRCLEAR
  CONTROL DISABLE hDlg, %IDC_BUTNPRINT
  XPRINT ATTACH CHOOSE, TRIM$(sTitle)     ' Show the Print dialog (COMDLG32)
  CONTROL ENABLE hDlg, %IDC_BUTNPRINT

  IF ERR = 0 AND LEN(XPRINT$) THEN        ' On success
      x1 = (@pd.mL) * (@pd.pXppi / 25.40) - @pd.lm  ' user-defined left margin
      y1 = (@pd.mT) * (@pd.pYppi / 25.40) - @pd.tm  ' user-defined top margin

      FONT NEW @pd.FontName, @pd.FontSize, @pd.FontType TO hFont
      XPRINT SET FONT hFont

      GOSUB PrintPageHeader             ' Begin by printing a header

      '------------------------------------------------------------------------
      FOR c = 1 TO UBOUND(sArray)
          XPRINT GET POS TO x, y        ' Check position
          IF c MOD (@pd.lpp+1) = 0 THEN ' When all lines-per-page has been printed,
              XPRINT FORMFEED           ' eject the paper and start a new page.
              GOSUB PrintPageHeader     ' Print header part
          ELSE
              XPRINT SET POS (x1, y)    ' else simply set position
          END IF
          XPRINT sArray(c)              ' and print line
      NEXT
      '------------------------------------------------------------------------

      XPRINT CLOSE                      ' End printjob and detach the printer

      FONT END hFont
  END IF

EXIT SUB

'--------------------------------------------------------------------
PrintPageHeader:
  INCR pgNum      ' print a centered header at top of the page
  sHeader = sTitle  + "  |  Page" + STR$(pgNum) + "/" + FORMAT$(@pd.PageCount)
  sHeader = sHeader + "  |  " + DATE$ + ", " + TIME$
  XPRINT TEXT SIZE sHeader TO w1, h1
  XPRINT SET POS ((@pd.pw - w1) / 2, (y1 - h1) / 2)
  XPRINT sHeader
  y = y1
  XPRINT SET POS (x1, y) ' set position for actual text
RETURN

END SUB


'====================================================================
SUB PrintToScreen (BYVAL hClient AS DWORD, sArray() AS STRING)
'--------------------------------------------------------------------
' Preview control setup
'--------------------------------------------------------------------
  LOCAL x, y, dw, dh, w, h, doFlag AS LONG
  LOCAL pd AS PreviewData PTR

  '------------------------------------------------------------------
  ' Turn off redraw for smoother action
  '------------------------------------------------------------------
  DIALOG SEND hClient, %WM_SETREDRAW, 0, 0

  '------------------------------------------------------------------
  DIALOG GET USER hClient, 2 TO pd    ' Get preview data
  IF pd = 0 THEN EXIT SUB             ' Ensure we got something

  '------------------------------------------------------------------
ShowPreviewStart:
  DIALOG GET CLIENT hClient TO dw, dh ' client size

  IF @pd.zoom = 0 THEN                ' Auto-size mode
      w = dw - 14                     ' Allow some space left/right
      h = dh - 14                     ' Allow some space above/below
  ELSE
      w = (@pd.zoom / 100) * (@pd.pw * (@pd.gXppi / @pd.pXppi))
      h = (@pd.zoom / 100) * (@pd.ph * (@pd.gYppi / @pd.pYppi))
  END IF

  IF @pd.orient = 2 THEN              ' If Landscape mode
      @pd.gw = w                      ' Graphic width
      @pd.gh = w * @pd.ph/@pd.pw      ' Graphic proportional height
      IF @pd.gh > h THEN              ' If Graphic height > max height
          @pd.gh = h                  ' Resize to max height
          @pd.gw = h * @pd.pw/@pd.ph  ' New proportional width
      END IF
  ELSE                                ' Else Portrait mode (default)
      @pd.gh = h                      ' Graphic proportional height
      @pd.gw = h * @pd.pw/@pd.ph      ' Graphic proportional width
      IF @pd.gw > w THEN              ' If Graphic width > max width
          @pd.gw = w                  ' Resize to max width
          @pd.gh = w * @pd.ph/@pd.pw  ' New proportional height
      END IF
  END IF
  x = MAX&(7, (dw - @pd.gw) / 2)      ' Horizontally centered
  y = MAX&(7, (dh - @pd.gh) / 2)      ' Vertically centered

  '------------------------------------------------------------------
  ' SetScrollSize, then repeat the calculations + SetScrollSize
  ' to ensure correct view in case scrollbars were added/removed.
  '------------------------------------------------------------------
  SetScrollSize hClient, @pd.gw + x + 7, @pd.gh + y + 7
  IF doFlag = 0 THEN
      doFlag = 1
      GOTO ShowPreviewStart
  END IF

  '------------------------------------------------------------------
  ' Destroy and re-create the Graphic control with new size
  '------------------------------------------------------------------
  CONTROL KILL hClient, %IDC_GRAPHICVIEW
  CONTROL ADD GRAPHIC, hClient, %IDC_GRAPHICVIEW, "", x, y, @pd.gw, @pd.gh, %SS_NOTIFY
  GRAPHIC ATTACH hClient, %IDC_GRAPHICVIEW, REDRAW
  GRAPHIC COLOR @pd.clrWindowText, @pd.clrWindow
  GRAPHIC CLEAR

  @pd.zoomX = @pd.gw / (@pd.pw * (@pd.gXppi / @pd.pXppi)) ' horizontal zoom factor
  @pd.zoomY = @pd.gh / (@pd.ph * (@pd.gYppi / @pd.pYppi)) ' vertical zoom factor

  '------------------------------------------------------------------
  ' Calculate margin positions
  '------------------------------------------------------------------
  @pd.gx1 = @pd.mL * (@pd.gXppi / 25.40) * @pd.zoomX
  @pd.gy1 = @pd.mT * (@pd.gYppi / 25.40) * @pd.zoomY
  @pd.gx2 = @pd.gw - (@pd.mR * (@pd.gXppi / 25.40) * @pd.zoomX) - 2 ' Adjust for 2-pixel shadow
  @pd.gy2 = @pd.gh - (@pd.mB * (@pd.gYppi / 25.40) * @pd.zoomY) - 2

  '------------------------------------------------------------------
  ' Stretch Graphic buffer to the new Graphic view control
  '------------------------------------------------------------------
  GRAPHIC STRETCH hClient, %IDC_GRAPHICBUFF, _
                 (0, 0)-(@pd.gpw-1, @pd.gph-1) TO (0, 0)-(@pd.gw-1, @pd.gh-1)

  '------------------------------------------------------------------
  ' Draw margins, using a dotted line
  '------------------------------------------------------------------
  GRAPHIC WIDTH 1
  GRAPHIC STYLE 2
  GRAPHIC LINE (@pd.gx1, 6)-(@pd.gx1, @pd.gh-6), @pd.clrShadow  ' Left margin
  GRAPHIC LINE (@pd.gx2, 6)-(@pd.gx2, @pd.gh-6), @pd.clrShadow  ' Right margin
  GRAPHIC LINE (6, @pd.gy1)-(@pd.gw-6, @pd.gy1), @pd.clrShadow  ' Top margin
  GRAPHIC LINE (6, @pd.gy2)-(@pd.gw-6, @pd.gy2), @pd.clrShadow  ' Bottom margin

  '------------------------------------------------------------------
  ' Draw borders and shadows
  '------------------------------------------------------------------
  GRAPHIC WIDTH 2
  GRAPHIC STYLE 1
  GRAPHIC BOX (1,1)-(@pd.gw-2, @pd.gh-2), 0, @pd.clrBorder        ' Blue-ish border
  GRAPHIC LINE (@pd.gw-1, 2)-(@pd.gw-1, @pd.gh), @pd.clrDkShadow  ' Right
  GRAPHIC LINE (2, @pd.gh-1)-(@pd.gw, @pd.gh-1), @pd.clrDkShadow  ' Bottom
  GRAPHIC LINE (@pd.gw-1, -1)-(@pd.gw-1, 2), @pd.clrShadow  ' Top/right corner adjustment
  GRAPHIC LINE (-1, @pd.gh-1)-(2, @pd.gh-1), @pd.clrShadow  ' Left/bottom corner adjustment

  '------------------------------------------------------------------
  SetScrollPosHV hClient                    ' Set scroll position
  DIALOG SEND hClient, %WM_SETREDRAW, 1, 0  ' Turn on redraw
  DIALOG REDRAW hClient                     ' and redraw
  GRAPHIC REDRAW

END SUB


'====================================================================
SUB SetScrollPosHV (BYVAL hClient AS DWORD)
'--------------------------------------------------------------------
' Set scroll position
'--------------------------------------------------------------------
  LOCAL rc AS RECT, si AS SCROLLINFO
  LOCAL pd AS PreviewData PTR
  STATIC oldW AS LONG, oldH AS LONG

  '------------------------------------------------------------------
  DIALOG GET USER hClient, 2 TO pd        ' Get preview data

  '------------------------------------------------------------------
  GetClientRect hClient, rc   ' scroll window's client size, in pixels

  '------------------------------------------------------------------
  ' Set horizontal scroll
  '------------------------------------------------------------------
  si.cbSize = SIZEOF(SCROLLINFO)       ' fill in SCROLLINFO members
  si.fMask  = %SIF_ALL                 ' Grab all scrollbar info
  GetScrollInfo hClient, %SB_HORZ, si

  IF oldW = 0 THEN oldW = @pd.gw + 13
  rc.nLeft = MAX&(0, rc.nRight - oldW)

  IF @pd.zoom = 0 OR ((@pd.pt.x + si.nPos - rc.nLeft) < oldW / 3) THEN
      si.nPos = 0
  ELSEIF (@pd.pt.x + si.nPos - rc.nLeft) > oldW / 3 * 2 THEN
      si.nPos = si.nMax - si.nPage
  ELSE
      si.nPos = MIN&(si.nMax - si.nPage, ((@pd.pt.x + si.nPos) / oldW) * si.nMax - @pd.pt.x)
  END IF
  oldW = @pd.gw + 13

  si.fMask  = %SIF_POS                    ' Set scrollbar pos
  SetScrollInfo hClient, %SB_HORZ, si, 1  ' set horizontal scrollbar info
  ScrollWindow hClient, -si.nPos, 0, BYVAL %NULL, BYVAL %NULL

  '------------------------------------------------------------------
  ' Set vertical scroll
  '------------------------------------------------------------------
  si.fMask  = %SIF_ALL                 ' Grab all scrollbar info
  GetScrollInfo hClient, %SB_VERT, si

  IF oldH = 0 THEN oldH = @pd.gh + 13
  rc.nTop = MAX&(0, rc.nBottom - oldH)

  IF @pd.zoom = 0 OR (si.nPos = 0 AND ((@pd.pt.y + si.nPos - rc.nTop) < oldH / 3)) THEN
      si.nPos = 0
  ELSEIF (@pd.pt.y + si.nPos - rc.nTop) > oldH / 3 * 2 THEN
      si.nPos = si.nMax - si.nPage
  ELSE
      si.nPos = MIN&(si.nMax - si.nPage, ((@pd.pt.y + si.nPos) / oldH) * si.nMax - @pd.pt.y)
  END IF
  oldH = @pd.gh + 13

  si.fMask  = %SIF_POS                    ' Set scrollbar pos
  SetScrollInfo hClient, %SB_VERT, si, 1  ' set vertical scrollbar info
  ScrollWindow hClient, 0, -si.nPos, BYVAL %NULL, BYVAL %NULL

  '------------------------------------------------------------------
  ' Reset cursor position
  '------------------------------------------------------------------
  IF @pd.zoom = 0 THEN
      @pd.pt.x = 0
      @pd.pt.y = 0
  END IF

END SUB


'====================================================================
SUB SetScrollSize (BYVAL hClient AS DWORD, BYVAL w AS LONG, BYVAL h AS LONG)
'--------------------------------------------------------------------
' Set scroll area
'--------------------------------------------------------------------
  LOCAL rc AS RECT, si AS SCROLLINFO
  '------------------------------------------------------------------
  GetClientRect hClient, rc   ' scroll window's client size, in pixels

  '------------------------------------------------------------------
  ' adjust scrolling zone to include eventual scrollbars
  '------------------------------------------------------------------
  IF (GetWindowLong(hClient, %GWL_STYLE) AND %WS_HSCROLL) = 0 THEN
      IF w > rc.nRight  THEN
          rc.nBottom = rc.nBottom - GetSystemMetrics(%SM_CYHSCROLL)
      END IF
  ELSE
      IF w < rc.nRight - GetSystemMetrics(%SM_CXVSCROLL) THEN
          rc.nBottom = rc.nBottom + GetSystemMetrics(%SM_CYHSCROLL)
      END IF
  END IF

  IF (GetWindowLong(hClient, %GWL_STYLE) AND %WS_VSCROLL) = 0 THEN
      IF h > rc.nBottom THEN
          rc.nRight = rc.nRight - GetSystemMetrics(%SM_CXVSCROLL)
      END IF
  ELSE
      IF h < rc.nBottom - GetSystemMetrics(%WS_HSCROLL) THEN
          rc.nRight = rc.nRight + GetSystemMetrics(%SM_CXVSCROLL)
      END IF
  END IF

  '------------------------------------------------------------------
  h = MAX&(h, rc.nBottom) - 1
  w = MAX&(w, rc.nRight) - 1

  si.cbSize = SIZEOF(SCROLLINFO)          ' fill in SCROLLINFO members
  si.fMask  = %SIF_PAGE OR %SIF_RANGE     ' need this mask

  '------------------------------------------------------------------
  GetScrollInfo hClient, %SB_HORZ, si
  si.nMin   = 0                           ' min scroll pos
  si.nMax   = w                           ' max scroll pos
  si.nPage  = rc.nRight                   ' page width
  SetScrollInfo hClient, %SB_HORZ, si, 1  ' set horizontal scrollbar info
  '------------------------------------------------------------------
  GetScrollInfo hClient, %SB_VERT, si
  si.nMin   = 0                           ' min scroll pos
  si.nMax   = h                           ' max scroll pos
  si.nPage  = rc.nBottom                  ' page height
  SetScrollInfo hClient, %SB_VERT, si, 1  ' set vertical scrollbar info

END SUB


'====================================================================
CALLBACK FUNCTION ViewPortProc() AS LONG
'--------------------------------------------------------------------
' View port - Child dialog's (ScrollWindow's) callback procedure
'--------------------------------------------------------------------
  SELECT CASE AS LONG CB.MSG
  CASE %WM_HSCROLL    : FUNCTION = vp_OnHScroll (CB.HNDL, CB.WPARAM, CB.LPARAM)
  CASE %WM_VSCROLL    : FUNCTION = vp_OnVScroll (CB.HNDL, CB.WPARAM, CB.LPARAM)
  END SELECT

END FUNCTION


'====================================================================
FUNCTION vp_OnHScroll (BYVAL hWnd AS DWORD, _
                       BYVAL wParam AS DWORD, _
                       BYVAL lParam AS LONG) AS LONG
'--------------------------------------------------------------------
  ' %WM_HSCROLL handler
  '------------------------------------------------------------------
  LOCAL oldPosY AS LONG
  LOCAL si AS SCROLLINFO

  si.cbSize = SIZEOF(SCROLLINFO)
  si.fMask  = %SIF_POS OR %SIF_PAGE OR %SIF_RANGE
  GetScrollInfo hWnd, %SB_HORZ, si
  oldPosY   = si.nPos

  SELECT CASE LOWRD(wParam)
      CASE %SB_LINELEFT   : si.nPos = si.nPos - 10
      CASE %SB_PAGELEFT   : si.nPos = si.nPos - si.nPage
      CASE %SB_LINERIGHT  : si.nPos = si.nPos + 10
      CASE %SB_PAGERIGHT  : si.nPos = si.nPos + si.nPage
      CASE %SB_THUMBTRACK : si.nPos = HIWRD(wParam)
      CASE ELSE           : EXIT FUNCTION
  END SELECT

  ' Ensure pos is within range
  si.nPos = MAX&(si.nMin, MIN&(si.nPos, si.nMax - si.nPage))

  ' Update the scroll bar and scroll window
  si.fMask  = %SIF_POS
  SetScrollInfo hWnd, %SB_HORZ, si, 1
  ScrollWindow hWnd, oldPosY - si.nPos, 0, BYVAL %NULL, BYVAL %NULL

END FUNCTION


'====================================================================
FUNCTION vp_OnMouseWheel (BYVAL hWnd AS DWORD, _
                          BYVAL wParam AS DWORD, _
                          BYVAL lParam AS LONG) AS LONG
'--------------------------------------------------------------------
  ' %WM_MOUSEWHEEL handler
  '------------------------------------------------------------------

  LOCAL zDelta AS INTEGER, pvParam AS LONG
  zDelta = HIWRD(wParam)

  SystemParametersInfo %SPI_GETWHEELSCROLLLINES, 0, pvParam, 0
  IF pvParam = 0 THEN pvParam = 1

  IF (LOWRD(wParam) AND %MK_MBUTTON) = %MK_MBUTTON THEN 'horizontal scroll
      IF pvParam > 0 THEN
          IF zDelta > 0 THEN               'scroll to the left  (Arrow left)
              SendMessage hWnd, %WM_HSCROLL, MAKLNG(%SB_LINELEFT, 0), 0
          ELSE                             'scroll to the right (Arrow right
              SendMessage hWnd, %WM_HSCROLL, MAKLNG(%SB_LINERIGHT, 0), 0
          END IF
      ELSE
          IF zDelta > 0 THEN               'scroll to the left  (Page left)
              SendMessage hWnd, %WM_HSCROLL, MAKLNG(%SB_PAGELEFT, 0), 0
          ELSE                             'scroll to the right (Page right
              SendMessage hWnd, %WM_HSCROLL, MAKLNG(%SB_PAGERIGHT, 0), 0
          END IF
      END IF

  ELSE                                      'vertical scroll
      IF pvParam > 0 THEN
          IF zDelta > 0 THEN                'scroll upwards    (Arrow up)
              FOR pvParam = 1 TO pvParam
                  SendMessage hWnd, %WM_VSCROLL, MAKLNG(%SB_LINEUP, 0), 0
              NEXT
          ELSE                              'scroll downwards  (Arrow down)
              FOR pvParam = 1 TO pvParam
                  SendMessage hWnd, %WM_VSCROLL, MAKLNG(%SB_LINEDOWN, 0), 0
              NEXT
          END IF
      ELSE
          IF zDelta > 0 THEN                'scroll upwards    (Page up)
              SendMessage hWnd, %WM_VSCROLL, MAKLNG(%SB_PAGEUP, 0), 0
          ELSE                              'scroll downwards  (Page down)
              SendMessage hWnd, %WM_VSCROLL, MAKLNG(%SB_PAGEDOWN, 0), 0
          END IF
      END IF
  END IF
END FUNCTION


'====================================================================
FUNCTION vp_OnVScroll (BYVAL hWnd AS DWORD, _
                       BYVAL wParam AS DWORD, _
                       BYVAL lParam AS LONG) AS LONG
'--------------------------------------------------------------------
  ' %WM_VSCROLL handler
  '------------------------------------------------------------------
  LOCAL oldPosY AS LONG
  LOCAL si AS SCROLLINFO

  si.cbSize = SIZEOF(SCROLLINFO)
  si.fMask  = %SIF_POS OR %SIF_PAGE OR %SIF_RANGE
  GetScrollInfo hWnd, %SB_VERT, si
  oldPosY   = si.nPos

  SELECT CASE LOWRD(wParam)
      CASE %SB_LINEDOWN   : si.nPos = si.nPos + 10
      CASE %SB_PAGEDOWN   : si.nPos = si.nPos + si.nPage
      CASE %SB_LINEUP     : si.nPos = si.nPos - 10
      CASE %SB_PAGEUP     : si.nPos = si.nPos - si.nPage
      CASE %SB_THUMBTRACK : si.nPos = HIWRD(wParam)
      CASE ELSE           : EXIT FUNCTION
  END SELECT

  ' Ensure pos is within range
  si.nPos = MAX&(si.nMin, MIN&(si.nPos, si.nMax - si.nPage))

  ' Update the scroll bar and scroll window
  si.fMask  = %SIF_POS
  SetScrollInfo hWnd, %SB_VERT, si, 1
  ScrollWindow hWnd, 0, oldPosY - si.nPos, BYVAL %NULL, BYVAL %NULL

END FUNCTION
