'====================================================================
'
'  TextView.bas for PowerBASIC Compiler for Windows
'  Copyright (c) 2005 - 2008 PowerBASIC, Inc.
'  All Rights Reserved.
'
'--------------------------------------------------------------------
'
'  Demo for the Print Preview dialog include file, Preview.inc.
'
'  The program has been XP Theme enabled via a file called xptheme.xml,
'  which is embedded through the program's resource file (TEXTVIEW.RC).
'
'====================================================================
' Declares
'----------------------------------------------------------
#COMPILER PBWIN 9
#COMPILE EXE
#RESOURCE "TEXTVIEW.PBR"  ' Resource file - see TEXTVIEW.RC
'----------------------------------------------------------
%USEMACROS = 1            ' Use Macros for leanest result
#INCLUDE "COMDLG32.INC"   ' Common dialog declares
#INCLUDE "PREVIEW.INC"    ' Preview dialog include
'----------------------------------------------------------
%IDC_BUTNOPEN   = 120     ' Control id's
%IDC_BUTNVIEW   = 121
%IDC_BUTNPAGE   = 122
%IDC_BUTNFONT   = 123
%IDC_TEXTBOX1   = 131
'----------------------------------------------------------
$PROGNAME = "TextView"     ' Program title string equate
'----------------------------------------------------------


'====================================================================
FUNCTION PBMAIN()
'--------------------------------------------------------------------
' Program entry point
'--------------------------------------------------------------------
  LOCAL hDlg AS LONG

  DIALOG NEW 0, $PROGNAME, , , 380, 240, %WS_CAPTION OR %WS_SYSMENU OR _
                %WS_MINIMIZEBOX OR %WS_MAXIMIZEBOX OR %WS_THICKFRAME TO hDlg

  DIALOG SET ICON hDlg, "APPICON"

  '------------------------------------------------------------------
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNOPEN,  "&Open",      2, 3, 50, 14
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNVIEW,  "&Preview",  52, 3, 50, 14
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNPAGE,  "&Margins", 106, 3, 50, 14
  CONTROL ADD BUTTON, hDlg, %IDC_BUTNFONT,  "&Font",    156, 3, 50, 14
  CONTROL ADD BUTTON, hDlg, %IDCANCEL,      "E&xit",    210, 3, 50, 14

  CONTROL ADD TEXTBOX, hDlg, %IDC_TEXTBOX1, "", 0, 20, 201, 101, _
              %ES_MULTILINE OR %ES_WANTRETURN OR %WS_HSCROLL OR _
              %WS_VSCROLL OR %WS_TABSTOP, %WS_EX_CLIENTEDGE


  '------------------------------------------------------------------
  ' Note: the standard TextBox has a theoretical 64 KB max text size
  ' limit in Win95/98, but usually it's even less than that. To set
  ' as high limit as possible, %EM_SETLIMITTEXT can be used, with
  ' wParam = 0 to set the maximum text limit. In Windows XP there
  ' should be no practical text size limitations at all..
  '------------------------------------------------------------------
  CONTROL SEND hDlg, %IDC_TEXTBOX1, %EM_SETLIMITTEXT, 0, 0

  '------------------------------------------------------------------
  DIALOG SHOW MODAL hDlg, CALL DlgProc

END FUNCTION


'====================================================================
CALLBACK FUNCTION DlgProc() AS LONG
'--------------------------------------------------------------------
' Main Dialog procedure
'--------------------------------------------------------------------
  LOCAL h, w, x, y AS LONG
  LOCAL sText, sTitle AS STRING
  LOCAL hNewFont AS LONG
  STATIC hFont AS LONG

  SELECT CASE AS LONG CBMSG
  CASE %WM_INITDIALOG  ' <- Received right before the dialog is shown
      '------------------------------------------------------------------
      ' The main program is responsible for maintaining a Static or Global
      ' PreviewData variable and calling InitPreviewDlg before the Print
      ' Preview dialog in Preview.inc is used.
      '------------------------------------------------------------------
      STATIC pd AS PreviewData  ' <- Defined in Preview.inc
      InitPreviewDlg pd         ' <- Located in Preview.inc

      '------------------------------------------------------------------
      ' Create and set an initial fixed-width font in the TextBox.
      '------------------------------------------------------------------
      FONT NEW "Courier New", 10 TO hFont
      CONTROL SET FONT CB.HNDL, %IDC_TEXTBOX1, hFont

  CASE %WM_COMMAND
      '----------------------------------------------------
      ' Messages from the controls are handled here.
      ' Example: Button clicks generates a %BN_CLICKED message
      '          and menu clicks generates the value 1.
      '----------------------------------------------------
      SELECT CASE AS LONG CBCTL
      CASE %IDC_BUTNOPEN  ' "Open" button was clicked
          IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
              sText = OpenFileProc(CBHNDL)
              IF LEN(sText) THEN
                  CONTROL SET TEXT CBHNDL, %IDC_TEXTBOX1, sText
              END IF
          END IF

      CASE %IDC_BUTNVIEW  ' "Preview" button was clicked
          IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
              ' Grab TextBox contents, dialog title and show Print Preview
              CONTROL GET TEXT CBHNDL, %IDC_TEXTBOX1 TO sText
              DIALOG GET TEXT CBHNDL TO sTitle
              DlgPreview CBHNDL, sTitle, sText, pd
          END IF

      CASE %IDC_BUTNPAGE
          IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
              PageSetupDlgProc CBHNDL, pd
          END IF

      CASE %IDC_BUTNFONT
          IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
              IF SelectFontProc (CBHNDL, hNewFont, pd) THEN
                  ' Assing the new font to the textbox
                  CONTROL SET FONT CB.HNDL, %IDC_TEXTBOX1, hNewFont
                  ' Destory the previously used font
                  FONT END hFont
                  ' Set the new font as the current font
                  hFont = hNewFont
              END IF
          END IF

      CASE %IDCANCEL      ' "Exit" button or the Esc-key was pressed
          IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
              DIALOG END CBHNDL, 0
          END IF
      END SELECT


  CASE %WM_SIZE  ' is sent on resize - auto-size the TextBox
      IF CBWPARAM <> %SIZE_MINIMIZED THEN  ' avoid minimized state
          w = LO(WORD, CBLPARAM)      ' dialog client area's width in pixels
          h = HI(WORD, CBLPARAM)      ' dialog client area's height in pixels
          DIALOG PIXELS CBHNDL, w, h TO UNITS w, h  ' convert to dialog units
          CONTROL GET LOC CBHNDL, %IDC_TEXTBOX1 TO x, y      ' we need top pos
          CONTROL SET SIZE CBHNDL, %IDC_TEXTBOX1, w, h - y   ' auto-size..
      END IF

  END SELECT

END FUNCTION


'====================================================================
FUNCTION GetFile (BYVAL sFile AS STRING) AS STRING
'--------------------------------------------------------------------
' Open and return given file's contents as a string
'--------------------------------------------------------------------
  LOCAL ff   AS LONG
  LOCAL sBuf AS STRING

  IF LEN(DIR$(sFile)) = 0 THEN
      MSGBOX "The file does not exist.", %MB_TASKMODAL, "Error"
      EXIT FUNCTION
  END IF

  ff = FREEFILE
  OPEN sFile FOR BINARY AS ff LEN = 8192
      IF ERR THEN  ' Always trap ev. errors on file open actions
          MSGBOX ERROR$(ERR), %MB_TASKMODAL, "Error"
          RESET : ERRCLEAR : EXIT FUNCTION
      END IF
      GET$ ff, LOF(ff), sBuf
  CLOSE ff

  FUNCTION = sBuf

END FUNCTION


'====================================================================
FUNCTION OpenFileProc (BYVAL hDlg AS DWORD) AS STRING
'--------------------------------------------------------------------
' Open file dialog procedure
'--------------------------------------------------------------------
  LOCAL  dwStyle AS DWORD
  LOCAL  Buf     AS STRING
  LOCAL  sFile   AS STRING
  STATIC sPath   AS STRING  ' Use a Static path variable

  IF LEN(sPath) = 0 THEN sPath = CURDIR$
  sFile = "*.BAS;*.INC;*.RC"

  Buf = CHR$("PB code files (*.BAS, *.INC, *.RC)", 0, "*.BAS;*.INC;*.RC", 0,  _
             "Text files (*.TXT)", 0, "*.TXT", 0, _
             "All files (*.*)", 0, "*.*", 0)

  dwStyle = %OFN_EXPLORER OR %OFN_FILEMUSTEXIST OR %OFN_HIDEREADONLY

  DISPLAY OPENFILE hDlg, , , "", sPath, buf, "", "BAS", dwStyle TO sFile

  IF LEN(sFile) THEN
      ' Store the selected file's path in static sPath for next time
      sPath = PATHNAME$(PATH, sFile)
      ' Set file name to dialog caption
      DIALOG SET TEXT hDlg, $PROGNAME + " - " + sFile

      ' Grab and return the file's contents as a text string
      FUNCTION = GetFile(sFile)

  END IF

END FUNCTION


'====================================================================
FUNCTION PageSetupDlgProc (BYVAL hDlg AS DWORD, _
                           pd AS PreviewData) AS LONG
'--------------------------------------------------------------------
' Use COMDLG32's Page setup dialog for margin settings
'--------------------------------------------------------------------
  LOCAL psd AS PAGESETUPDLGA
  '------------------------------------------------------------------
  psd.lStructSize      = SIZEOF(psd)
  psd.hwndOwner        = hDlg
  psd.flags            = %PSD_MARGINS OR %PSD_DISABLEPAPER OR _
                         %PSD_DISABLEORIENTATION OR %PSD_DISABLEPRINTER

  '------------------------------------------------------------------
  ' The Preview dialog uses the more fine-grained Metric millimeters
  ' internally, so in US/Inch systems we need to convert the values.
  ' Note: The PageSetupDlg uses 1/1000 Inches (US) or 1/100 mm's (Metric) ..
  '------------------------------------------------------------------
  IF pd.locale = 1 THEN  ' Convert from mm. to Inches
      psd.rtMargin.nLeft   = pd.mL * 1000 / 25.4
      psd.rtMargin.nTop    = pd.mT * 1000 / 25.4
      psd.rtMargin.nRight  = pd.mR * 1000 / 25.4
      psd.rtMargin.nBottom = pd.mB * 1000 / 25.4
  ELSE
      psd.rtMargin.nLeft   = pd.mL * 100
      psd.rtMargin.nTop    = pd.mT * 100
      psd.rtMargin.nRight  = pd.mR * 100
      psd.rtMargin.nBottom = pd.mB * 100
  END IF

  '------------------------------------------------------------------
  ' Show the PageSetup dialog
  '------------------------------------------------------------------
  IF PageSetupDlg(psd) THEN
      IF pd.locale = 1 THEN  ' Convert from Inches to mm.
          pd.mL = psd.rtMargin.nLeft   / 1000 * 25.4
          pd.mT = psd.rtMargin.nTop    / 1000 * 25.4
          pd.mR = psd.rtMargin.nRight  / 1000 * 25.4
          pd.mB = psd.rtMargin.nBottom / 1000 * 25.4
      ELSE
          pd.mL = psd.rtMargin.nLeft   / 100
          pd.mT = psd.rtMargin.nTop    / 100
          pd.mR = psd.rtMargin.nRight  / 100
          pd.mB = psd.rtMargin.nBottom / 100
      END IF
      '----------------------------------------------------------
      FUNCTION = 1
  END IF
  IF psd.hDevMode  THEN GlobalFree psd.hDevMode
  IF psd.hDevNames THEN GlobalFree psd.hDevNames

END FUNCTION


'====================================================================
FUNCTION SelectFontProc (BYVAL hDlg AS DWORD, hFont AS LONG, pd AS PreviewData) AS LONG

  LOCAL  newFontName  AS STRING
  LOCAL  newFontSize  AS LONG
  LOCAL  newFontStyle AS LONG

  DISPLAY FONT hDlg, , , pd.FontName, pd.FontSize, pd.FontType, %CF_BOTH OR %CF_FORCEFONTEXIST OR %CF_INITTOLOGFONTSTRUCT OR %CF_NOSCRIPTSEL _
      TO newFontName, newFontSize, newFontStyle

  IF LEN(newFontName) THEN
     pd.FontName = newFontName
     pd.FontSize = newFontSize
     pd.FontType = newFontStyle

    FONT NEW newFontName, newFontSize, newFontStyle TO hFont
    FUNCTION = %TRUE
  END IF

END FUNCTION
